# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...types import balance_list_params, balance_create_params, balance_update_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from .transactions import (
    TransactionsResource,
    AsyncTransactionsResource,
    TransactionsResourceWithRawResponse,
    AsyncTransactionsResourceWithRawResponse,
    TransactionsResourceWithStreamingResponse,
    AsyncTransactionsResourceWithStreamingResponse,
)
from ..._base_client import AsyncPaginator, make_request_options
from ...types.balance import Balance

__all__ = ["BalancesResource", "AsyncBalancesResource"]


class BalancesResource(SyncAPIResource):
    @cached_property
    def transactions(self) -> TransactionsResource:
        return TransactionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> BalancesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return BalancesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BalancesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return BalancesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        account_id: str,
        currency: str,
        end_date: Union[str, datetime],
        start_date: Union[str, datetime],
        balance_draw_down_description: str | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        consumptions_accounting_product_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        fees_accounting_product_id: str | NotGiven = NOT_GIVEN,
        line_item_types: List[
            Literal[
                "STANDING_CHARGE", "USAGE", "MINIMUM_SPEND", "COUNTER_RUNNING_TOTAL_CHARGE", "COUNTER_ADJUSTMENT_DEBIT"
            ]
        ]
        | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        overage_description: str | NotGiven = NOT_GIVEN,
        overage_surcharge_percent: float | NotGiven = NOT_GIVEN,
        product_ids: List[str] | NotGiven = NOT_GIVEN,
        rollover_amount: float | NotGiven = NOT_GIVEN,
        rollover_end_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Balance:
        """
        Create a new Balance for the given end customer Account.

        This endpoint allows you to create a new Balance for a specific end customer
        Account. The Balance details should be provided in the request body.

        Args:
          account_id: The unique identifier (UUID) for the end customer Account.

          currency: The currency code used for the Balance amount. For example: USD, GBP or EUR.

          end_date: The date _(in ISO 8601 format)_ after which the Balance will no longer be active
              for the Account.

              **Note:** You can use the `rolloverEndDate` request parameter to define an
              extended grace period for continued draw-down against the Balance if any amount
              remains when the specified `endDate` is reached.

          start_date: The date _(in ISO 8601 format)_ when the Balance becomes active.

          balance_draw_down_description: A description for the bill line items for draw-down charges against the Balance.
              _(Optional)._

          code: Unique short code for the Balance.

          consumptions_accounting_product_id: Optional Product ID this Balance Consumptions should be attributed to for
              accounting purposes

          description: A description of the Balance.

          fees_accounting_product_id: Optional Product ID this Balance Fees should be attributed to for accounting
              purposes

          line_item_types: Specify the line item charge types that can draw-down at billing against the
              Balance amount. Options are:

              - `"MINIMUM_SPEND"`
              - `"STANDING_CHARGE"`
              - `"USAGE"`
              - `"COUNTER_RUNNING_TOTAL_CHARGE"`
              - `"COUNTER_ADJUSTMENT_DEBIT"`

              **NOTE:** If no charge types are specified, by default _all types_ can draw-down
              against the Balance amount at billing.

          name: The official name for the Balance.

          overage_description: A description for Bill line items overage charges.

          overage_surcharge_percent: Define a surcharge level, as a percentage of regular usage rating, applied to
              overages _(usage charges that exceed the Balance amount)_. For example, if the
              regular usage rate is $10 per unit of usage consumed and
              `overageSurchargePercent` is set at 10%, then any usage charged above the
              original Balance amount is charged at $11 per unit of usage.

          product_ids: Specify the Products whose consumption charges due at billing can be drawn-down
              against the Balance amount.

              **Note:** If you don't specify any Products for Balance draw-down, by default
              the consumption charges for any Product the Account consumes will be drawn-down
              against the Balance amount.

          rollover_amount: The maximum amount that can be carried over past the Balance end date for
              draw-down at billing if there is any unused Balance amount when the end date is
              reached. Works with `rolloverEndDate` to define the amount and duration of a
              Balance "grace period". _(Optional)_

              **Notes:**

              - If you leave `rolloverAmount` empty and only enter a `rolloverEndDate`, any
                amount left over after the Balance end date is reached will be drawn-down
                against up to the specified `rolloverEndDate`.
              - You must enter a `rolloverEndDate`. If you only enter a `rolloverAmount`
                without entering a `rolloverEndDate`, you'll receive an error when trying to
                create or update the Balance.
              - If you don't want to grant any grace period for outstanding Balance amounts,
                then do not use `rolloverAmount` and `rolloverEndDate`.

          rollover_end_date: The end date _(in ISO 8601 format)_ for the grace period during which unused
              Balance amounts can be carried over and drawn-down against at billing.

              **Note:** Use `rolloverAmount` if you want to specify a maximum amount that can
              be carried over and made available for draw-down.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/balances",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "currency": currency,
                    "end_date": end_date,
                    "start_date": start_date,
                    "balance_draw_down_description": balance_draw_down_description,
                    "code": code,
                    "consumptions_accounting_product_id": consumptions_accounting_product_id,
                    "description": description,
                    "fees_accounting_product_id": fees_accounting_product_id,
                    "line_item_types": line_item_types,
                    "name": name,
                    "overage_description": overage_description,
                    "overage_surcharge_percent": overage_surcharge_percent,
                    "product_ids": product_ids,
                    "rollover_amount": rollover_amount,
                    "rollover_end_date": rollover_end_date,
                    "version": version,
                },
                balance_create_params.BalanceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Balance:
        """
        Retrieve a specific Balance.

        This endpoint returns the details of the specified Balance.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/balances/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        account_id: str,
        currency: str,
        end_date: Union[str, datetime],
        start_date: Union[str, datetime],
        balance_draw_down_description: str | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        consumptions_accounting_product_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        fees_accounting_product_id: str | NotGiven = NOT_GIVEN,
        line_item_types: List[
            Literal[
                "STANDING_CHARGE", "USAGE", "MINIMUM_SPEND", "COUNTER_RUNNING_TOTAL_CHARGE", "COUNTER_ADJUSTMENT_DEBIT"
            ]
        ]
        | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        overage_description: str | NotGiven = NOT_GIVEN,
        overage_surcharge_percent: float | NotGiven = NOT_GIVEN,
        product_ids: List[str] | NotGiven = NOT_GIVEN,
        rollover_amount: float | NotGiven = NOT_GIVEN,
        rollover_end_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Balance:
        """
        Update a specific Balance.

        This endpoint allows you to update the details of a specific Balance. The
        updated Balance details should be provided in the request body.

        Args:
          account_id: The unique identifier (UUID) for the end customer Account.

          currency: The currency code used for the Balance amount. For example: USD, GBP or EUR.

          end_date: The date _(in ISO 8601 format)_ after which the Balance will no longer be active
              for the Account.

              **Note:** You can use the `rolloverEndDate` request parameter to define an
              extended grace period for continued draw-down against the Balance if any amount
              remains when the specified `endDate` is reached.

          start_date: The date _(in ISO 8601 format)_ when the Balance becomes active.

          balance_draw_down_description: A description for the bill line items for draw-down charges against the Balance.
              _(Optional)._

          code: Unique short code for the Balance.

          consumptions_accounting_product_id: Optional Product ID this Balance Consumptions should be attributed to for
              accounting purposes

          description: A description of the Balance.

          fees_accounting_product_id: Optional Product ID this Balance Fees should be attributed to for accounting
              purposes

          line_item_types: Specify the line item charge types that can draw-down at billing against the
              Balance amount. Options are:

              - `"MINIMUM_SPEND"`
              - `"STANDING_CHARGE"`
              - `"USAGE"`
              - `"COUNTER_RUNNING_TOTAL_CHARGE"`
              - `"COUNTER_ADJUSTMENT_DEBIT"`

              **NOTE:** If no charge types are specified, by default _all types_ can draw-down
              against the Balance amount at billing.

          name: The official name for the Balance.

          overage_description: A description for Bill line items overage charges.

          overage_surcharge_percent: Define a surcharge level, as a percentage of regular usage rating, applied to
              overages _(usage charges that exceed the Balance amount)_. For example, if the
              regular usage rate is $10 per unit of usage consumed and
              `overageSurchargePercent` is set at 10%, then any usage charged above the
              original Balance amount is charged at $11 per unit of usage.

          product_ids: Specify the Products whose consumption charges due at billing can be drawn-down
              against the Balance amount.

              **Note:** If you don't specify any Products for Balance draw-down, by default
              the consumption charges for any Product the Account consumes will be drawn-down
              against the Balance amount.

          rollover_amount: The maximum amount that can be carried over past the Balance end date for
              draw-down at billing if there is any unused Balance amount when the end date is
              reached. Works with `rolloverEndDate` to define the amount and duration of a
              Balance "grace period". _(Optional)_

              **Notes:**

              - If you leave `rolloverAmount` empty and only enter a `rolloverEndDate`, any
                amount left over after the Balance end date is reached will be drawn-down
                against up to the specified `rolloverEndDate`.
              - You must enter a `rolloverEndDate`. If you only enter a `rolloverAmount`
                without entering a `rolloverEndDate`, you'll receive an error when trying to
                create or update the Balance.
              - If you don't want to grant any grace period for outstanding Balance amounts,
                then do not use `rolloverAmount` and `rolloverEndDate`.

          rollover_end_date: The end date _(in ISO 8601 format)_ for the grace period during which unused
              Balance amounts can be carried over and drawn-down against at billing.

              **Note:** Use `rolloverAmount` if you want to specify a maximum amount that can
              be carried over and made available for draw-down.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/balances/{id}",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "currency": currency,
                    "end_date": end_date,
                    "start_date": start_date,
                    "balance_draw_down_description": balance_draw_down_description,
                    "code": code,
                    "consumptions_accounting_product_id": consumptions_accounting_product_id,
                    "description": description,
                    "fees_accounting_product_id": fees_accounting_product_id,
                    "line_item_types": line_item_types,
                    "name": name,
                    "overage_description": overage_description,
                    "overage_surcharge_percent": overage_surcharge_percent,
                    "product_ids": product_ids,
                    "rollover_amount": rollover_amount,
                    "rollover_end_date": rollover_end_date,
                    "version": version,
                },
                balance_update_params.BalanceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: str | NotGiven = NOT_GIVEN,
        end_date_end: str | NotGiven = NOT_GIVEN,
        end_date_start: str | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[Balance]:
        """
        Retrieve a list of all Balances for your Organization.

        This endpoint returns a list of all Balances associated with your organization.
        You can filter the Balances by the end customer's Account UUID and end dates,
        and paginate through them using the `pageSize` and `nextToken` parameters.

        Args:
          account_id: The unique identifier (UUID) for the end customer's account.

          end_date_end: Only include Balances with end dates earlier than this date.

          end_date_start: Only include Balances with end dates equal to or later than this date.

          next_token: The `nextToken` for retrieving the next page of Balances. It is used to fetch
              the next page of Balances in a paginated list.

          page_size: The maximum number of Balances to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/balances",
            page=SyncCursor[Balance],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "end_date_end": end_date_end,
                        "end_date_start": end_date_start,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    balance_list_params.BalanceListParams,
                ),
            ),
            model=Balance,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Balance:
        """
        Delete a specific Balance.

        This endpoint allows you to delete a specific Balance with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/balances/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )


class AsyncBalancesResource(AsyncAPIResource):
    @cached_property
    def transactions(self) -> AsyncTransactionsResource:
        return AsyncTransactionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBalancesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBalancesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBalancesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncBalancesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        account_id: str,
        currency: str,
        end_date: Union[str, datetime],
        start_date: Union[str, datetime],
        balance_draw_down_description: str | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        consumptions_accounting_product_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        fees_accounting_product_id: str | NotGiven = NOT_GIVEN,
        line_item_types: List[
            Literal[
                "STANDING_CHARGE", "USAGE", "MINIMUM_SPEND", "COUNTER_RUNNING_TOTAL_CHARGE", "COUNTER_ADJUSTMENT_DEBIT"
            ]
        ]
        | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        overage_description: str | NotGiven = NOT_GIVEN,
        overage_surcharge_percent: float | NotGiven = NOT_GIVEN,
        product_ids: List[str] | NotGiven = NOT_GIVEN,
        rollover_amount: float | NotGiven = NOT_GIVEN,
        rollover_end_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Balance:
        """
        Create a new Balance for the given end customer Account.

        This endpoint allows you to create a new Balance for a specific end customer
        Account. The Balance details should be provided in the request body.

        Args:
          account_id: The unique identifier (UUID) for the end customer Account.

          currency: The currency code used for the Balance amount. For example: USD, GBP or EUR.

          end_date: The date _(in ISO 8601 format)_ after which the Balance will no longer be active
              for the Account.

              **Note:** You can use the `rolloverEndDate` request parameter to define an
              extended grace period for continued draw-down against the Balance if any amount
              remains when the specified `endDate` is reached.

          start_date: The date _(in ISO 8601 format)_ when the Balance becomes active.

          balance_draw_down_description: A description for the bill line items for draw-down charges against the Balance.
              _(Optional)._

          code: Unique short code for the Balance.

          consumptions_accounting_product_id: Optional Product ID this Balance Consumptions should be attributed to for
              accounting purposes

          description: A description of the Balance.

          fees_accounting_product_id: Optional Product ID this Balance Fees should be attributed to for accounting
              purposes

          line_item_types: Specify the line item charge types that can draw-down at billing against the
              Balance amount. Options are:

              - `"MINIMUM_SPEND"`
              - `"STANDING_CHARGE"`
              - `"USAGE"`
              - `"COUNTER_RUNNING_TOTAL_CHARGE"`
              - `"COUNTER_ADJUSTMENT_DEBIT"`

              **NOTE:** If no charge types are specified, by default _all types_ can draw-down
              against the Balance amount at billing.

          name: The official name for the Balance.

          overage_description: A description for Bill line items overage charges.

          overage_surcharge_percent: Define a surcharge level, as a percentage of regular usage rating, applied to
              overages _(usage charges that exceed the Balance amount)_. For example, if the
              regular usage rate is $10 per unit of usage consumed and
              `overageSurchargePercent` is set at 10%, then any usage charged above the
              original Balance amount is charged at $11 per unit of usage.

          product_ids: Specify the Products whose consumption charges due at billing can be drawn-down
              against the Balance amount.

              **Note:** If you don't specify any Products for Balance draw-down, by default
              the consumption charges for any Product the Account consumes will be drawn-down
              against the Balance amount.

          rollover_amount: The maximum amount that can be carried over past the Balance end date for
              draw-down at billing if there is any unused Balance amount when the end date is
              reached. Works with `rolloverEndDate` to define the amount and duration of a
              Balance "grace period". _(Optional)_

              **Notes:**

              - If you leave `rolloverAmount` empty and only enter a `rolloverEndDate`, any
                amount left over after the Balance end date is reached will be drawn-down
                against up to the specified `rolloverEndDate`.
              - You must enter a `rolloverEndDate`. If you only enter a `rolloverAmount`
                without entering a `rolloverEndDate`, you'll receive an error when trying to
                create or update the Balance.
              - If you don't want to grant any grace period for outstanding Balance amounts,
                then do not use `rolloverAmount` and `rolloverEndDate`.

          rollover_end_date: The end date _(in ISO 8601 format)_ for the grace period during which unused
              Balance amounts can be carried over and drawn-down against at billing.

              **Note:** Use `rolloverAmount` if you want to specify a maximum amount that can
              be carried over and made available for draw-down.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/balances",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "currency": currency,
                    "end_date": end_date,
                    "start_date": start_date,
                    "balance_draw_down_description": balance_draw_down_description,
                    "code": code,
                    "consumptions_accounting_product_id": consumptions_accounting_product_id,
                    "description": description,
                    "fees_accounting_product_id": fees_accounting_product_id,
                    "line_item_types": line_item_types,
                    "name": name,
                    "overage_description": overage_description,
                    "overage_surcharge_percent": overage_surcharge_percent,
                    "product_ids": product_ids,
                    "rollover_amount": rollover_amount,
                    "rollover_end_date": rollover_end_date,
                    "version": version,
                },
                balance_create_params.BalanceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Balance:
        """
        Retrieve a specific Balance.

        This endpoint returns the details of the specified Balance.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/balances/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        account_id: str,
        currency: str,
        end_date: Union[str, datetime],
        start_date: Union[str, datetime],
        balance_draw_down_description: str | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        consumptions_accounting_product_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        fees_accounting_product_id: str | NotGiven = NOT_GIVEN,
        line_item_types: List[
            Literal[
                "STANDING_CHARGE", "USAGE", "MINIMUM_SPEND", "COUNTER_RUNNING_TOTAL_CHARGE", "COUNTER_ADJUSTMENT_DEBIT"
            ]
        ]
        | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        overage_description: str | NotGiven = NOT_GIVEN,
        overage_surcharge_percent: float | NotGiven = NOT_GIVEN,
        product_ids: List[str] | NotGiven = NOT_GIVEN,
        rollover_amount: float | NotGiven = NOT_GIVEN,
        rollover_end_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Balance:
        """
        Update a specific Balance.

        This endpoint allows you to update the details of a specific Balance. The
        updated Balance details should be provided in the request body.

        Args:
          account_id: The unique identifier (UUID) for the end customer Account.

          currency: The currency code used for the Balance amount. For example: USD, GBP or EUR.

          end_date: The date _(in ISO 8601 format)_ after which the Balance will no longer be active
              for the Account.

              **Note:** You can use the `rolloverEndDate` request parameter to define an
              extended grace period for continued draw-down against the Balance if any amount
              remains when the specified `endDate` is reached.

          start_date: The date _(in ISO 8601 format)_ when the Balance becomes active.

          balance_draw_down_description: A description for the bill line items for draw-down charges against the Balance.
              _(Optional)._

          code: Unique short code for the Balance.

          consumptions_accounting_product_id: Optional Product ID this Balance Consumptions should be attributed to for
              accounting purposes

          description: A description of the Balance.

          fees_accounting_product_id: Optional Product ID this Balance Fees should be attributed to for accounting
              purposes

          line_item_types: Specify the line item charge types that can draw-down at billing against the
              Balance amount. Options are:

              - `"MINIMUM_SPEND"`
              - `"STANDING_CHARGE"`
              - `"USAGE"`
              - `"COUNTER_RUNNING_TOTAL_CHARGE"`
              - `"COUNTER_ADJUSTMENT_DEBIT"`

              **NOTE:** If no charge types are specified, by default _all types_ can draw-down
              against the Balance amount at billing.

          name: The official name for the Balance.

          overage_description: A description for Bill line items overage charges.

          overage_surcharge_percent: Define a surcharge level, as a percentage of regular usage rating, applied to
              overages _(usage charges that exceed the Balance amount)_. For example, if the
              regular usage rate is $10 per unit of usage consumed and
              `overageSurchargePercent` is set at 10%, then any usage charged above the
              original Balance amount is charged at $11 per unit of usage.

          product_ids: Specify the Products whose consumption charges due at billing can be drawn-down
              against the Balance amount.

              **Note:** If you don't specify any Products for Balance draw-down, by default
              the consumption charges for any Product the Account consumes will be drawn-down
              against the Balance amount.

          rollover_amount: The maximum amount that can be carried over past the Balance end date for
              draw-down at billing if there is any unused Balance amount when the end date is
              reached. Works with `rolloverEndDate` to define the amount and duration of a
              Balance "grace period". _(Optional)_

              **Notes:**

              - If you leave `rolloverAmount` empty and only enter a `rolloverEndDate`, any
                amount left over after the Balance end date is reached will be drawn-down
                against up to the specified `rolloverEndDate`.
              - You must enter a `rolloverEndDate`. If you only enter a `rolloverAmount`
                without entering a `rolloverEndDate`, you'll receive an error when trying to
                create or update the Balance.
              - If you don't want to grant any grace period for outstanding Balance amounts,
                then do not use `rolloverAmount` and `rolloverEndDate`.

          rollover_end_date: The end date _(in ISO 8601 format)_ for the grace period during which unused
              Balance amounts can be carried over and drawn-down against at billing.

              **Note:** Use `rolloverAmount` if you want to specify a maximum amount that can
              be carried over and made available for draw-down.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/balances/{id}",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "currency": currency,
                    "end_date": end_date,
                    "start_date": start_date,
                    "balance_draw_down_description": balance_draw_down_description,
                    "code": code,
                    "consumptions_accounting_product_id": consumptions_accounting_product_id,
                    "description": description,
                    "fees_accounting_product_id": fees_accounting_product_id,
                    "line_item_types": line_item_types,
                    "name": name,
                    "overage_description": overage_description,
                    "overage_surcharge_percent": overage_surcharge_percent,
                    "product_ids": product_ids,
                    "rollover_amount": rollover_amount,
                    "rollover_end_date": rollover_end_date,
                    "version": version,
                },
                balance_update_params.BalanceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: str | NotGiven = NOT_GIVEN,
        end_date_end: str | NotGiven = NOT_GIVEN,
        end_date_start: str | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[Balance, AsyncCursor[Balance]]:
        """
        Retrieve a list of all Balances for your Organization.

        This endpoint returns a list of all Balances associated with your organization.
        You can filter the Balances by the end customer's Account UUID and end dates,
        and paginate through them using the `pageSize` and `nextToken` parameters.

        Args:
          account_id: The unique identifier (UUID) for the end customer's account.

          end_date_end: Only include Balances with end dates earlier than this date.

          end_date_start: Only include Balances with end dates equal to or later than this date.

          next_token: The `nextToken` for retrieving the next page of Balances. It is used to fetch
              the next page of Balances in a paginated list.

          page_size: The maximum number of Balances to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/balances",
            page=AsyncCursor[Balance],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "end_date_end": end_date_end,
                        "end_date_start": end_date_start,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    balance_list_params.BalanceListParams,
                ),
            ),
            model=Balance,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Balance:
        """
        Delete a specific Balance.

        This endpoint allows you to delete a specific Balance with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/balances/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )


class BalancesResourceWithRawResponse:
    def __init__(self, balances: BalancesResource) -> None:
        self._balances = balances

        self.create = to_raw_response_wrapper(
            balances.create,
        )
        self.retrieve = to_raw_response_wrapper(
            balances.retrieve,
        )
        self.update = to_raw_response_wrapper(
            balances.update,
        )
        self.list = to_raw_response_wrapper(
            balances.list,
        )
        self.delete = to_raw_response_wrapper(
            balances.delete,
        )

    @cached_property
    def transactions(self) -> TransactionsResourceWithRawResponse:
        return TransactionsResourceWithRawResponse(self._balances.transactions)


class AsyncBalancesResourceWithRawResponse:
    def __init__(self, balances: AsyncBalancesResource) -> None:
        self._balances = balances

        self.create = async_to_raw_response_wrapper(
            balances.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            balances.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            balances.update,
        )
        self.list = async_to_raw_response_wrapper(
            balances.list,
        )
        self.delete = async_to_raw_response_wrapper(
            balances.delete,
        )

    @cached_property
    def transactions(self) -> AsyncTransactionsResourceWithRawResponse:
        return AsyncTransactionsResourceWithRawResponse(self._balances.transactions)


class BalancesResourceWithStreamingResponse:
    def __init__(self, balances: BalancesResource) -> None:
        self._balances = balances

        self.create = to_streamed_response_wrapper(
            balances.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            balances.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            balances.update,
        )
        self.list = to_streamed_response_wrapper(
            balances.list,
        )
        self.delete = to_streamed_response_wrapper(
            balances.delete,
        )

    @cached_property
    def transactions(self) -> TransactionsResourceWithStreamingResponse:
        return TransactionsResourceWithStreamingResponse(self._balances.transactions)


class AsyncBalancesResourceWithStreamingResponse:
    def __init__(self, balances: AsyncBalancesResource) -> None:
        self._balances = balances

        self.create = async_to_streamed_response_wrapper(
            balances.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            balances.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            balances.update,
        )
        self.list = async_to_streamed_response_wrapper(
            balances.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            balances.delete,
        )

    @cached_property
    def transactions(self) -> AsyncTransactionsResourceWithStreamingResponse:
        return AsyncTransactionsResourceWithStreamingResponse(self._balances.transactions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal

import httpx

from ..types import organization_config_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.organization_config_response import OrganizationConfigResponse
from ..types.shared_params.currency_conversion import CurrencyConversion

__all__ = ["OrganizationConfigResource", "AsyncOrganizationConfigResource"]


class OrganizationConfigResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OrganizationConfigResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return OrganizationConfigResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OrganizationConfigResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return OrganizationConfigResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationConfigResponse:
        """
        Retrieve the Organization-wide configuration details.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get(
            f"/organizations/{org_id}/organizationconfig",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationConfigResponse,
        )

    def update(
        self,
        *,
        org_id: str | None = None,
        currency: str,
        day_epoch: str,
        days_before_bill_due: int,
        month_epoch: str,
        timezone: str,
        week_epoch: str,
        year_epoch: str,
        auto_approve_bills_grace_period: int | NotGiven = NOT_GIVEN,
        auto_approve_bills_grace_period_unit: str | NotGiven = NOT_GIVEN,
        auto_generate_statement_mode: Literal["NONE", "JSON", "JSON_AND_CSV"] | NotGiven = NOT_GIVEN,
        bill_prefix: str | NotGiven = NOT_GIVEN,
        commitment_fee_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        consolidate_bills: bool | NotGiven = NOT_GIVEN,
        credit_application_order: List[Literal["PREPAYMENT", "BALANCE"]] | NotGiven = NOT_GIVEN,
        currency_conversions: Iterable[CurrencyConversion] | NotGiven = NOT_GIVEN,
        default_statement_definition_id: str | NotGiven = NOT_GIVEN,
        external_invoice_date: str | NotGiven = NOT_GIVEN,
        minimum_spend_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        scheduled_bill_interval: float | NotGiven = NOT_GIVEN,
        sequence_start_number: int | NotGiven = NOT_GIVEN,
        standing_charge_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        suppressed_empty_bills: bool | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationConfigResponse:
        """
        Update the Organization-wide configuration details.

        Args:
          currency:
              The currency code for the Organization. For example: USD, GBP, or EUR:

              - This defines the _billing currency_ for the Organization. You can override
                this by selecting a different billing currency at individual Account level.
              - You must first define the currencies you want to use in your Organization. See
                the [Currency](https://www.m3ter.com/docs/api#tag/Currency) section in this
                API Reference.

              **Note:** If you use a different currency as the _pricing currency_ for Plans to
              set charge rates for Product consumption by an Account, you must define a
              currency conversion rate from the pricing currency to the billing currency
              before you run billing for the Account, otherwise billing will fail. See below
              for the `currencyConversions` request parameter.

          day_epoch: Optional setting that defines the billing cycle date for Accounts that are
              billed daily. Defines the date of the first Bill:

              - For example, suppose the Plan you attach to an Account is configured for daily
                billing frequency and will apply to the Account from January 1st, 2022 until
                June 30th, 2022. If you set a `dayEpoch` date of January 2nd, 2022, then the
                first Bill is created for the Account on that date and subsequent Bills are
                created for the Account each day following through to the end of the billing
                service period.
              - The date is in ISO-8601 format.

          days_before_bill_due: Enter the number of days after the Bill generation date that you want to show on
              Bills as the due date.

              **Note:** If you define `daysBeforeBillDue` at individual Account level, this
              will take precedence over any `daysBeforeBillDue` setting defined at
              Organization level.

          month_epoch: Optional setting that defines the billing cycle date for Accounts that are
              billed monthly. Defines the date of the first Bill and then acts as reference
              for when subsequent Bills are created for the Account:

              - For example, suppose the Plan you attach to an Account is configured for
                monthly billing frequency and will apply to the Account from January 1st, 2022
                until June 30th, 2022. If you set a `monthEpoch` date of January 15th, 2022,
                then the first Bill is created for the Account on that date and subsequent
                Bills are created for the Account on the 15th of each month following through
                to the end of the billing service period - February 15th, March 15th, and so
                on.
              - The date is in ISO-8601 format.

          timezone: Sets the timezone for the Organization.

          week_epoch: Optional setting that defines the billing cycle date for Accounts that are
              billed weekly. Defines the date of the first Bill and then acts as reference for
              when subsequent Bills are created for the Account:

              - For example, suppose the Plan you attach to an Account is configured for
                weekly billing frequency and will apply to the Account from January 1st, 2022
                until June 30th, 2022. If you set a `weekEpoch` date of January 15th, 2022,
                which falls on a Saturday, then the first Bill is created for the Account on
                that date and subsequent Bills are created for the Account on Saturday of each
                week following through to the end of the billing service period.
              - The date is in ISO-8601 format.

          year_epoch: Optional setting that defines the billing cycle date for Accounts that are
              billed yearly. Defines the date of the first Bill and then acts as reference for
              when subsequent Bills are created for the Account:

              - For example, suppose the Plan you attach to an Account is configured for
                yearly billing frequency and will apply to the Account from January 1st, 2022
                until January 15th, 2028. If you set a `yearEpoch` date of January 1st, 2023,
                then the first Bill is created for the Account on that date and subsequent
                Bills are created for the Account on January 1st of each year following
                through to the end of the billing service period - January 1st, 2023, January
                1st, 2024 and so on.
              - The date is in ISO-8601 format.

          auto_approve_bills_grace_period: Grace period before bills are auto-approved. Used in combination with
              `autoApproveBillsGracePeriodUnit` parameter.

              **Note:** When used in combination with `autoApproveBillsGracePeriodUnit`
              enables auto-approval of Bills for Organization, which occurs when the specified
              time period has elapsed after Bill generation.

          auto_approve_bills_grace_period_unit: Time unit of grace period before bills are auto-approved. Used in combination
              with `autoApproveBillsGracePeriod` parameter. Allowed options are MINUTES,
              HOURS, or DAYS.

              **Note:** When used in combination with `autoApproveBillsGracePeriod` enables
              auto-approval of Bills for Organization, which occurs when the specified time
              period has elapsed after Bill generation.

          auto_generate_statement_mode: Specify whether to auto-generate statements once Bills are _approved_ or
              _locked_. It will not auto-generate if a bill is in _pending_ state.

              The default value is **None**.

              - **None**. Statements will not be auto-generated.
              - **JSON**. Statements are auto-generated in JSON format.
              - **JSON and CSV**. Statements are auto-generated in both JSON and CSV formats.

          bill_prefix: Prefix to be used for sequential invoice numbers. This will be combined with the
              `sequenceStartNumber`.

              **NOTES:**

              - If you do not define a `billPrefix`, a default will be used in the Console for
                the Bill **REFERENCE** number. This default will concatenate **INV-** with the
                last four characters of the `billId`.
              - If you do not define a `billPrefix`, the Bill response schema for API calls
                that retrieve Bill data will not contain a `sequentialInvoiceNumber`.

          commitment_fee_bill_in_advance: Boolean setting to specify whether commitments _(prepayments)_ are billed in
              advance at the start of each billing period, or billed in arrears at the end of
              each billing period.

              - **TRUE** - bill in advance _(start of each billing period)_.
              - **FALSE** - bill in arrears _(end of each billing period)_.

          consolidate_bills: Boolean setting to consolidate different billing frequencies onto the same bill.

              - **TRUE** - consolidate different billing frequencies onto the same bill.
              - **FALSE** - bills are not consolidated.

          credit_application_order: Define the order in which any Prepayment or Balance amounts on Accounts are to
              be drawn-down against for billing. Four options:

              - `"PREPAYMENT","BALANCE"`. Draw-down against Prepayment credit before Balance
                credit.
              - `"BALANCE","PREPAYMENT"`. Draw-down against Balance credit before Prepayment
                credit.
              - `"PREPAYMENT"`. Only draw-down against Prepayment credit.
              - `"BALANCE"`. Only draw-down against Balance credit.

              **NOTES:**

              - You can override this Organization-level setting for `creditApplicationOrder`
                at the level of an individual Account.
              - If the Account belongs to a Parent/Child Account hierarchy, then the
                `creditApplicationOrder` settings are not available, and the draw-down order
                defaults always to Prepayment then Balance order.

          currency_conversions:
              Define currency conversion rates from _pricing currency_ to _billing currency_:

              - You can use the `currency` request parameter with this call to define the
                billing currency for your Organization - see above.
              - You can also define a billing currency at the individual Account level and
                this will override the Organization billing currency.
              - A Plan used to set Product consumption charge rates on an Account might use a
                different pricing currency. At billing, charges are calculated in the pricing
                currency and then converted into billing currency amounts to appear on Bills.
                If you haven't defined a currency conversion rate from pricing to billing
                currency, billing will fail for the Account.

          default_statement_definition_id: Organization level default `statementDefinitionId` to be used when there is no
              statement definition linked to the account.

              Statement definitions are used to generate bill statements, which are
              informative backing sheets to invoices.

          external_invoice_date: Date to use for the invoice date. Allowed values are `FIRST_DAY_OF_NEXT_PERIOD`
              or `LAST_DAY_OF_ARREARS`.

          minimum_spend_bill_in_advance: Boolean setting to specify whether minimum spend amounts are billed in advance
              at the start of each billing period, or billed in arrears at the end of each
              billing period.

              - **TRUE** - bill in advance _(start of each billing period)_.
              - **FALSE** - bill in arrears _(end of each billing period)_.

          scheduled_bill_interval: Sets the required interval for updating bills. It is an optional parameter that
              can be set as:

              - **For portions of an hour (minutes)**. Two options: **0.25** (15 minutes) and
                **0.5** (30 minutes).
              - **For full hours.** Enter **1** for every hour, **2** for every two hours, and
                so on. Eight options: **1**, **2**, **3**, **4**, **6**, **8**, **12**, or
                **24**.
              - **Default.** The default is **0**, which disables scheduling.

          sequence_start_number: The starting number to be used for sequential invoice numbers. This will be
              combined with the `billPrefix`.

              For example, if you define `billPrefix` to be **INVOICE-** and you set the
              `seqenceStartNumber` as **100**, the first Bill created after updating your
              Organization Configuration will have a `sequentialInvoiceNumber` assigned of
              **INVOICE-101**. Subsequent Bills created will be numbered in time sequence for
              their initial creation date/time.

          standing_charge_bill_in_advance: Boolean setting to specify whether the standing charge is billed in advance at
              the start of each billing period, or billed in arrears at the end of each
              billing period.

              - **TRUE** - bill in advance _(start of each billing period)_.
              - **FALSE** - bill in arrears _(end of each billing period)_.

          suppressed_empty_bills: Boolean setting that supresses generating bills that have no line items.

              - **TRUE** - prevents generating bills with no line items.
              - **FALSE** - bills are still generated even when they have no line items.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._put(
            f"/organizations/{org_id}/organizationconfig",
            body=maybe_transform(
                {
                    "currency": currency,
                    "day_epoch": day_epoch,
                    "days_before_bill_due": days_before_bill_due,
                    "month_epoch": month_epoch,
                    "timezone": timezone,
                    "week_epoch": week_epoch,
                    "year_epoch": year_epoch,
                    "auto_approve_bills_grace_period": auto_approve_bills_grace_period,
                    "auto_approve_bills_grace_period_unit": auto_approve_bills_grace_period_unit,
                    "auto_generate_statement_mode": auto_generate_statement_mode,
                    "bill_prefix": bill_prefix,
                    "commitment_fee_bill_in_advance": commitment_fee_bill_in_advance,
                    "consolidate_bills": consolidate_bills,
                    "credit_application_order": credit_application_order,
                    "currency_conversions": currency_conversions,
                    "default_statement_definition_id": default_statement_definition_id,
                    "external_invoice_date": external_invoice_date,
                    "minimum_spend_bill_in_advance": minimum_spend_bill_in_advance,
                    "scheduled_bill_interval": scheduled_bill_interval,
                    "sequence_start_number": sequence_start_number,
                    "standing_charge_bill_in_advance": standing_charge_bill_in_advance,
                    "suppressed_empty_bills": suppressed_empty_bills,
                    "version": version,
                },
                organization_config_update_params.OrganizationConfigUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationConfigResponse,
        )


class AsyncOrganizationConfigResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOrganizationConfigResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOrganizationConfigResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOrganizationConfigResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncOrganizationConfigResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationConfigResponse:
        """
        Retrieve the Organization-wide configuration details.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._get(
            f"/organizations/{org_id}/organizationconfig",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationConfigResponse,
        )

    async def update(
        self,
        *,
        org_id: str | None = None,
        currency: str,
        day_epoch: str,
        days_before_bill_due: int,
        month_epoch: str,
        timezone: str,
        week_epoch: str,
        year_epoch: str,
        auto_approve_bills_grace_period: int | NotGiven = NOT_GIVEN,
        auto_approve_bills_grace_period_unit: str | NotGiven = NOT_GIVEN,
        auto_generate_statement_mode: Literal["NONE", "JSON", "JSON_AND_CSV"] | NotGiven = NOT_GIVEN,
        bill_prefix: str | NotGiven = NOT_GIVEN,
        commitment_fee_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        consolidate_bills: bool | NotGiven = NOT_GIVEN,
        credit_application_order: List[Literal["PREPAYMENT", "BALANCE"]] | NotGiven = NOT_GIVEN,
        currency_conversions: Iterable[CurrencyConversion] | NotGiven = NOT_GIVEN,
        default_statement_definition_id: str | NotGiven = NOT_GIVEN,
        external_invoice_date: str | NotGiven = NOT_GIVEN,
        minimum_spend_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        scheduled_bill_interval: float | NotGiven = NOT_GIVEN,
        sequence_start_number: int | NotGiven = NOT_GIVEN,
        standing_charge_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        suppressed_empty_bills: bool | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OrganizationConfigResponse:
        """
        Update the Organization-wide configuration details.

        Args:
          currency:
              The currency code for the Organization. For example: USD, GBP, or EUR:

              - This defines the _billing currency_ for the Organization. You can override
                this by selecting a different billing currency at individual Account level.
              - You must first define the currencies you want to use in your Organization. See
                the [Currency](https://www.m3ter.com/docs/api#tag/Currency) section in this
                API Reference.

              **Note:** If you use a different currency as the _pricing currency_ for Plans to
              set charge rates for Product consumption by an Account, you must define a
              currency conversion rate from the pricing currency to the billing currency
              before you run billing for the Account, otherwise billing will fail. See below
              for the `currencyConversions` request parameter.

          day_epoch: Optional setting that defines the billing cycle date for Accounts that are
              billed daily. Defines the date of the first Bill:

              - For example, suppose the Plan you attach to an Account is configured for daily
                billing frequency and will apply to the Account from January 1st, 2022 until
                June 30th, 2022. If you set a `dayEpoch` date of January 2nd, 2022, then the
                first Bill is created for the Account on that date and subsequent Bills are
                created for the Account each day following through to the end of the billing
                service period.
              - The date is in ISO-8601 format.

          days_before_bill_due: Enter the number of days after the Bill generation date that you want to show on
              Bills as the due date.

              **Note:** If you define `daysBeforeBillDue` at individual Account level, this
              will take precedence over any `daysBeforeBillDue` setting defined at
              Organization level.

          month_epoch: Optional setting that defines the billing cycle date for Accounts that are
              billed monthly. Defines the date of the first Bill and then acts as reference
              for when subsequent Bills are created for the Account:

              - For example, suppose the Plan you attach to an Account is configured for
                monthly billing frequency and will apply to the Account from January 1st, 2022
                until June 30th, 2022. If you set a `monthEpoch` date of January 15th, 2022,
                then the first Bill is created for the Account on that date and subsequent
                Bills are created for the Account on the 15th of each month following through
                to the end of the billing service period - February 15th, March 15th, and so
                on.
              - The date is in ISO-8601 format.

          timezone: Sets the timezone for the Organization.

          week_epoch: Optional setting that defines the billing cycle date for Accounts that are
              billed weekly. Defines the date of the first Bill and then acts as reference for
              when subsequent Bills are created for the Account:

              - For example, suppose the Plan you attach to an Account is configured for
                weekly billing frequency and will apply to the Account from January 1st, 2022
                until June 30th, 2022. If you set a `weekEpoch` date of January 15th, 2022,
                which falls on a Saturday, then the first Bill is created for the Account on
                that date and subsequent Bills are created for the Account on Saturday of each
                week following through to the end of the billing service period.
              - The date is in ISO-8601 format.

          year_epoch: Optional setting that defines the billing cycle date for Accounts that are
              billed yearly. Defines the date of the first Bill and then acts as reference for
              when subsequent Bills are created for the Account:

              - For example, suppose the Plan you attach to an Account is configured for
                yearly billing frequency and will apply to the Account from January 1st, 2022
                until January 15th, 2028. If you set a `yearEpoch` date of January 1st, 2023,
                then the first Bill is created for the Account on that date and subsequent
                Bills are created for the Account on January 1st of each year following
                through to the end of the billing service period - January 1st, 2023, January
                1st, 2024 and so on.
              - The date is in ISO-8601 format.

          auto_approve_bills_grace_period: Grace period before bills are auto-approved. Used in combination with
              `autoApproveBillsGracePeriodUnit` parameter.

              **Note:** When used in combination with `autoApproveBillsGracePeriodUnit`
              enables auto-approval of Bills for Organization, which occurs when the specified
              time period has elapsed after Bill generation.

          auto_approve_bills_grace_period_unit: Time unit of grace period before bills are auto-approved. Used in combination
              with `autoApproveBillsGracePeriod` parameter. Allowed options are MINUTES,
              HOURS, or DAYS.

              **Note:** When used in combination with `autoApproveBillsGracePeriod` enables
              auto-approval of Bills for Organization, which occurs when the specified time
              period has elapsed after Bill generation.

          auto_generate_statement_mode: Specify whether to auto-generate statements once Bills are _approved_ or
              _locked_. It will not auto-generate if a bill is in _pending_ state.

              The default value is **None**.

              - **None**. Statements will not be auto-generated.
              - **JSON**. Statements are auto-generated in JSON format.
              - **JSON and CSV**. Statements are auto-generated in both JSON and CSV formats.

          bill_prefix: Prefix to be used for sequential invoice numbers. This will be combined with the
              `sequenceStartNumber`.

              **NOTES:**

              - If you do not define a `billPrefix`, a default will be used in the Console for
                the Bill **REFERENCE** number. This default will concatenate **INV-** with the
                last four characters of the `billId`.
              - If you do not define a `billPrefix`, the Bill response schema for API calls
                that retrieve Bill data will not contain a `sequentialInvoiceNumber`.

          commitment_fee_bill_in_advance: Boolean setting to specify whether commitments _(prepayments)_ are billed in
              advance at the start of each billing period, or billed in arrears at the end of
              each billing period.

              - **TRUE** - bill in advance _(start of each billing period)_.
              - **FALSE** - bill in arrears _(end of each billing period)_.

          consolidate_bills: Boolean setting to consolidate different billing frequencies onto the same bill.

              - **TRUE** - consolidate different billing frequencies onto the same bill.
              - **FALSE** - bills are not consolidated.

          credit_application_order: Define the order in which any Prepayment or Balance amounts on Accounts are to
              be drawn-down against for billing. Four options:

              - `"PREPAYMENT","BALANCE"`. Draw-down against Prepayment credit before Balance
                credit.
              - `"BALANCE","PREPAYMENT"`. Draw-down against Balance credit before Prepayment
                credit.
              - `"PREPAYMENT"`. Only draw-down against Prepayment credit.
              - `"BALANCE"`. Only draw-down against Balance credit.

              **NOTES:**

              - You can override this Organization-level setting for `creditApplicationOrder`
                at the level of an individual Account.
              - If the Account belongs to a Parent/Child Account hierarchy, then the
                `creditApplicationOrder` settings are not available, and the draw-down order
                defaults always to Prepayment then Balance order.

          currency_conversions:
              Define currency conversion rates from _pricing currency_ to _billing currency_:

              - You can use the `currency` request parameter with this call to define the
                billing currency for your Organization - see above.
              - You can also define a billing currency at the individual Account level and
                this will override the Organization billing currency.
              - A Plan used to set Product consumption charge rates on an Account might use a
                different pricing currency. At billing, charges are calculated in the pricing
                currency and then converted into billing currency amounts to appear on Bills.
                If you haven't defined a currency conversion rate from pricing to billing
                currency, billing will fail for the Account.

          default_statement_definition_id: Organization level default `statementDefinitionId` to be used when there is no
              statement definition linked to the account.

              Statement definitions are used to generate bill statements, which are
              informative backing sheets to invoices.

          external_invoice_date: Date to use for the invoice date. Allowed values are `FIRST_DAY_OF_NEXT_PERIOD`
              or `LAST_DAY_OF_ARREARS`.

          minimum_spend_bill_in_advance: Boolean setting to specify whether minimum spend amounts are billed in advance
              at the start of each billing period, or billed in arrears at the end of each
              billing period.

              - **TRUE** - bill in advance _(start of each billing period)_.
              - **FALSE** - bill in arrears _(end of each billing period)_.

          scheduled_bill_interval: Sets the required interval for updating bills. It is an optional parameter that
              can be set as:

              - **For portions of an hour (minutes)**. Two options: **0.25** (15 minutes) and
                **0.5** (30 minutes).
              - **For full hours.** Enter **1** for every hour, **2** for every two hours, and
                so on. Eight options: **1**, **2**, **3**, **4**, **6**, **8**, **12**, or
                **24**.
              - **Default.** The default is **0**, which disables scheduling.

          sequence_start_number: The starting number to be used for sequential invoice numbers. This will be
              combined with the `billPrefix`.

              For example, if you define `billPrefix` to be **INVOICE-** and you set the
              `seqenceStartNumber` as **100**, the first Bill created after updating your
              Organization Configuration will have a `sequentialInvoiceNumber` assigned of
              **INVOICE-101**. Subsequent Bills created will be numbered in time sequence for
              their initial creation date/time.

          standing_charge_bill_in_advance: Boolean setting to specify whether the standing charge is billed in advance at
              the start of each billing period, or billed in arrears at the end of each
              billing period.

              - **TRUE** - bill in advance _(start of each billing period)_.
              - **FALSE** - bill in arrears _(end of each billing period)_.

          suppressed_empty_bills: Boolean setting that supresses generating bills that have no line items.

              - **TRUE** - prevents generating bills with no line items.
              - **FALSE** - bills are still generated even when they have no line items.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._put(
            f"/organizations/{org_id}/organizationconfig",
            body=await async_maybe_transform(
                {
                    "currency": currency,
                    "day_epoch": day_epoch,
                    "days_before_bill_due": days_before_bill_due,
                    "month_epoch": month_epoch,
                    "timezone": timezone,
                    "week_epoch": week_epoch,
                    "year_epoch": year_epoch,
                    "auto_approve_bills_grace_period": auto_approve_bills_grace_period,
                    "auto_approve_bills_grace_period_unit": auto_approve_bills_grace_period_unit,
                    "auto_generate_statement_mode": auto_generate_statement_mode,
                    "bill_prefix": bill_prefix,
                    "commitment_fee_bill_in_advance": commitment_fee_bill_in_advance,
                    "consolidate_bills": consolidate_bills,
                    "credit_application_order": credit_application_order,
                    "currency_conversions": currency_conversions,
                    "default_statement_definition_id": default_statement_definition_id,
                    "external_invoice_date": external_invoice_date,
                    "minimum_spend_bill_in_advance": minimum_spend_bill_in_advance,
                    "scheduled_bill_interval": scheduled_bill_interval,
                    "sequence_start_number": sequence_start_number,
                    "standing_charge_bill_in_advance": standing_charge_bill_in_advance,
                    "suppressed_empty_bills": suppressed_empty_bills,
                    "version": version,
                },
                organization_config_update_params.OrganizationConfigUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationConfigResponse,
        )


class OrganizationConfigResourceWithRawResponse:
    def __init__(self, organization_config: OrganizationConfigResource) -> None:
        self._organization_config = organization_config

        self.retrieve = to_raw_response_wrapper(
            organization_config.retrieve,
        )
        self.update = to_raw_response_wrapper(
            organization_config.update,
        )


class AsyncOrganizationConfigResourceWithRawResponse:
    def __init__(self, organization_config: AsyncOrganizationConfigResource) -> None:
        self._organization_config = organization_config

        self.retrieve = async_to_raw_response_wrapper(
            organization_config.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            organization_config.update,
        )


class OrganizationConfigResourceWithStreamingResponse:
    def __init__(self, organization_config: OrganizationConfigResource) -> None:
        self._organization_config = organization_config

        self.retrieve = to_streamed_response_wrapper(
            organization_config.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            organization_config.update,
        )


class AsyncOrganizationConfigResourceWithStreamingResponse:
    def __init__(self, organization_config: AsyncOrganizationConfigResource) -> None:
        self._organization_config = organization_config

        self.retrieve = async_to_streamed_response_wrapper(
            organization_config.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            organization_config.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CurrencyConversion"]

_CurrencyConversionReservedKeywords = TypedDict(
    "_CurrencyConversionReservedKeywords",
    {
        "from": str,
    },
    total=False,
)


class CurrencyConversion(_CurrencyConversionReservedKeywords, total=False):
    to: Required[str]
    """Currency to convert to. For example: USD."""

    multiplier: float
    """Conversion rate between currencies."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["JobListParams"]


class JobListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    date_created_end: Annotated[str, PropertyInfo(alias="dateCreatedEnd")]
    """Include only Job entities created before this date.

    Format: yyyy-MM-dd'T'HH:mm:ss'Z'
    """

    date_created_start: Annotated[str, PropertyInfo(alias="dateCreatedStart")]
    """Include only Job entities created on or after this date.

    Format: yyyy-MM-dd'T'HH:mm:ss'Z'
    """

    ids: List[str]
    """List Job entities for the given UUIDs"""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """nextToken for multi page retrievals"""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Number of Jobs to retrieve per page"""

    schedule_id: Annotated[str, PropertyInfo(alias="scheduleId")]
    """List Job entities for the schedule UUID"""

    status: Literal["PENDING", "RUNNING", "SUCCEEDED", "FAILED"]
    """List Job entities for the status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..types import (
    notification_configuration_list_params,
    notification_configuration_create_params,
    notification_configuration_update_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.notification_configuration_response import NotificationConfigurationResponse

__all__ = ["NotificationConfigurationsResource", "AsyncNotificationConfigurationsResource"]


class NotificationConfigurationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> NotificationConfigurationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return NotificationConfigurationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NotificationConfigurationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return NotificationConfigurationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        code: str,
        description: str,
        event_name: str,
        name: str,
        active: bool | NotGiven = NOT_GIVEN,
        always_fire_event: bool | NotGiven = NOT_GIVEN,
        calculation: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NotificationConfigurationResponse:
        """
        Create a new Notification for an Event.

        This endpoint enables you to create a new Event Notification for the specified
        Organization. You need to supply a request body with the details of the new
        Notification.

        Args:
          code: The short code for the Notification.

          description: The description for the Notification providing a brief overview of its purpose
              and functionality.

          event_name: The name of the _Event type_ that the Notification is based on. When an Event of
              this type occurs and any calculation built into the Notification evaluates to
              `True`, the Notification is triggered.

              **Note:** If the Notification is set to always fire, then the Notification will
              always be sent when the Event of the type it is based on occurs, and without any
              other conditions defined by a calculation having to be met.

          name: The name of the Notification.

          active: Boolean flag that sets the Notification as active or inactive. Only active
              Notifications are sent when triggered by the Event they are based on:

              - **TRUE** - set Notification as active.
              - **FALSE** - set Notification as inactive.

          always_fire_event: A Boolean flag indicating whether the Notification is always triggered,
              regardless of other conditions and omitting reference to any calculation. This
              means the Notification will be triggered simply by the Event it is based on
              occurring and with no further conditions having to be met.

              - **TRUE** - the Notification is always triggered and omits any reference to the
                calculation to check for other conditions being true before triggering the
                Notification.
              - **FALSE** - the Notification is only triggered when the Event it is based on
                occurs and any calculation is checked and all conditions defined by the
                calculation are met.

          calculation: A logical expression that that is evaluated to a Boolean. If it evaluates as
              `True`, a Notification for the Event is created and sent to the configured
              destination. Calculations can reference numeric, string, and boolean Event
              fields.

              See
              [Creating Calculations](https://www.m3ter.com/docs/guides/utilizing-events-and-notifications/key-concepts-and-relationships#creating-calculations)
              in the m3ter documentation for more information.

          version:
              The version number for the Notification:

              - **Create:** Not valid for initial insertion of new entity - _do not use for
                Create_. On initial Create, version is set at 1 and listed in the response.
              - **Update:** On Update, version is required and must match the existing version
                because a check is performed to ensure sequential versioning is preserved.
                Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/notifications/configurations",
            body=maybe_transform(
                {
                    "code": code,
                    "description": description,
                    "event_name": event_name,
                    "name": name,
                    "active": active,
                    "always_fire_event": always_fire_event,
                    "calculation": calculation,
                    "version": version,
                },
                notification_configuration_create_params.NotificationConfigurationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationConfigurationResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NotificationConfigurationResponse:
        """Retrieve the details of a specific Notification using its UUID.

        Includes the
        Event the Notification is based on, and any calculation referencing the Event's
        field and which defines further conditions that must be met to trigger the
        Notification when the Event occurs.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/notifications/configurations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationConfigurationResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        code: str,
        description: str,
        event_name: str,
        name: str,
        active: bool | NotGiven = NOT_GIVEN,
        always_fire_event: bool | NotGiven = NOT_GIVEN,
        calculation: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NotificationConfigurationResponse:
        """
        Update a Notification with the given UUID.

        This endpoint modifies the configuration details of an existing Notification.
        You can change the Event that triggers the Notification and/or update the
        conditions for sending the Notification.

        Args:
          code: The short code for the Notification.

          description: The description for the Notification providing a brief overview of its purpose
              and functionality.

          event_name: The name of the _Event type_ that the Notification is based on. When an Event of
              this type occurs and any calculation built into the Notification evaluates to
              `True`, the Notification is triggered.

              **Note:** If the Notification is set to always fire, then the Notification will
              always be sent when the Event of the type it is based on occurs, and without any
              other conditions defined by a calculation having to be met.

          name: The name of the Notification.

          active: Boolean flag that sets the Notification as active or inactive. Only active
              Notifications are sent when triggered by the Event they are based on:

              - **TRUE** - set Notification as active.
              - **FALSE** - set Notification as inactive.

          always_fire_event: A Boolean flag indicating whether the Notification is always triggered,
              regardless of other conditions and omitting reference to any calculation. This
              means the Notification will be triggered simply by the Event it is based on
              occurring and with no further conditions having to be met.

              - **TRUE** - the Notification is always triggered and omits any reference to the
                calculation to check for other conditions being true before triggering the
                Notification.
              - **FALSE** - the Notification is only triggered when the Event it is based on
                occurs and any calculation is checked and all conditions defined by the
                calculation are met.

          calculation: A logical expression that that is evaluated to a Boolean. If it evaluates as
              `True`, a Notification for the Event is created and sent to the configured
              destination. Calculations can reference numeric, string, and boolean Event
              fields.

              See
              [Creating Calculations](https://www.m3ter.com/docs/guides/utilizing-events-and-notifications/key-concepts-and-relationships#creating-calculations)
              in the m3ter documentation for more information.

          version:
              The version number for the Notification:

              - **Create:** Not valid for initial insertion of new entity - _do not use for
                Create_. On initial Create, version is set at 1 and listed in the response.
              - **Update:** On Update, version is required and must match the existing version
                because a check is performed to ensure sequential versioning is preserved.
                Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/notifications/configurations/{id}",
            body=maybe_transform(
                {
                    "code": code,
                    "description": description,
                    "event_name": event_name,
                    "name": name,
                    "active": active,
                    "always_fire_event": always_fire_event,
                    "calculation": calculation,
                    "version": version,
                },
                notification_configuration_update_params.NotificationConfigurationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationConfigurationResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        active: bool | NotGiven = NOT_GIVEN,
        event_name: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[NotificationConfigurationResponse]:
        """
        Retrieve a list of Event Notifications for the specified Organization.

        This endpoint retrieves a list of all Event Notifications for the Organization
        identified by its UUID. The list can be paginated for easier management. The
        list also supports filtering by parameters such as Notification UUID.

        Args:
          active: A Boolean flag indicating whether to retrieve only active or only inactive
              Notifications.

              - **TRUE** - only active Notifications are returned.
              - **FALSE** - only inactive Notifications are returned.

          event_name: Use this to filter the Notifications returned - only those Notifications that
              are based on the _Event type_ specified by `eventName` are returned.

          ids: A list of specific Notification UUIDs to retrieve.

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              Notifications in a paginated list.

          page_size: Specifies the maximum number of Notifications to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/notifications/configurations",
            page=SyncCursor[NotificationConfigurationResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "active": active,
                        "event_name": event_name,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    notification_configuration_list_params.NotificationConfigurationListParams,
                ),
            ),
            model=NotificationConfigurationResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NotificationConfigurationResponse:
        """
        Delete the Notification with the given UUID.

        This endpoint permanently removes a specified Notification and its
        configuration. This action cannot be undone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/notifications/configurations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationConfigurationResponse,
        )


class AsyncNotificationConfigurationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncNotificationConfigurationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNotificationConfigurationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNotificationConfigurationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncNotificationConfigurationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        code: str,
        description: str,
        event_name: str,
        name: str,
        active: bool | NotGiven = NOT_GIVEN,
        always_fire_event: bool | NotGiven = NOT_GIVEN,
        calculation: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NotificationConfigurationResponse:
        """
        Create a new Notification for an Event.

        This endpoint enables you to create a new Event Notification for the specified
        Organization. You need to supply a request body with the details of the new
        Notification.

        Args:
          code: The short code for the Notification.

          description: The description for the Notification providing a brief overview of its purpose
              and functionality.

          event_name: The name of the _Event type_ that the Notification is based on. When an Event of
              this type occurs and any calculation built into the Notification evaluates to
              `True`, the Notification is triggered.

              **Note:** If the Notification is set to always fire, then the Notification will
              always be sent when the Event of the type it is based on occurs, and without any
              other conditions defined by a calculation having to be met.

          name: The name of the Notification.

          active: Boolean flag that sets the Notification as active or inactive. Only active
              Notifications are sent when triggered by the Event they are based on:

              - **TRUE** - set Notification as active.
              - **FALSE** - set Notification as inactive.

          always_fire_event: A Boolean flag indicating whether the Notification is always triggered,
              regardless of other conditions and omitting reference to any calculation. This
              means the Notification will be triggered simply by the Event it is based on
              occurring and with no further conditions having to be met.

              - **TRUE** - the Notification is always triggered and omits any reference to the
                calculation to check for other conditions being true before triggering the
                Notification.
              - **FALSE** - the Notification is only triggered when the Event it is based on
                occurs and any calculation is checked and all conditions defined by the
                calculation are met.

          calculation: A logical expression that that is evaluated to a Boolean. If it evaluates as
              `True`, a Notification for the Event is created and sent to the configured
              destination. Calculations can reference numeric, string, and boolean Event
              fields.

              See
              [Creating Calculations](https://www.m3ter.com/docs/guides/utilizing-events-and-notifications/key-concepts-and-relationships#creating-calculations)
              in the m3ter documentation for more information.

          version:
              The version number for the Notification:

              - **Create:** Not valid for initial insertion of new entity - _do not use for
                Create_. On initial Create, version is set at 1 and listed in the response.
              - **Update:** On Update, version is required and must match the existing version
                because a check is performed to ensure sequential versioning is preserved.
                Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/notifications/configurations",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "description": description,
                    "event_name": event_name,
                    "name": name,
                    "active": active,
                    "always_fire_event": always_fire_event,
                    "calculation": calculation,
                    "version": version,
                },
                notification_configuration_create_params.NotificationConfigurationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationConfigurationResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NotificationConfigurationResponse:
        """Retrieve the details of a specific Notification using its UUID.

        Includes the
        Event the Notification is based on, and any calculation referencing the Event's
        field and which defines further conditions that must be met to trigger the
        Notification when the Event occurs.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/notifications/configurations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationConfigurationResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        code: str,
        description: str,
        event_name: str,
        name: str,
        active: bool | NotGiven = NOT_GIVEN,
        always_fire_event: bool | NotGiven = NOT_GIVEN,
        calculation: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NotificationConfigurationResponse:
        """
        Update a Notification with the given UUID.

        This endpoint modifies the configuration details of an existing Notification.
        You can change the Event that triggers the Notification and/or update the
        conditions for sending the Notification.

        Args:
          code: The short code for the Notification.

          description: The description for the Notification providing a brief overview of its purpose
              and functionality.

          event_name: The name of the _Event type_ that the Notification is based on. When an Event of
              this type occurs and any calculation built into the Notification evaluates to
              `True`, the Notification is triggered.

              **Note:** If the Notification is set to always fire, then the Notification will
              always be sent when the Event of the type it is based on occurs, and without any
              other conditions defined by a calculation having to be met.

          name: The name of the Notification.

          active: Boolean flag that sets the Notification as active or inactive. Only active
              Notifications are sent when triggered by the Event they are based on:

              - **TRUE** - set Notification as active.
              - **FALSE** - set Notification as inactive.

          always_fire_event: A Boolean flag indicating whether the Notification is always triggered,
              regardless of other conditions and omitting reference to any calculation. This
              means the Notification will be triggered simply by the Event it is based on
              occurring and with no further conditions having to be met.

              - **TRUE** - the Notification is always triggered and omits any reference to the
                calculation to check for other conditions being true before triggering the
                Notification.
              - **FALSE** - the Notification is only triggered when the Event it is based on
                occurs and any calculation is checked and all conditions defined by the
                calculation are met.

          calculation: A logical expression that that is evaluated to a Boolean. If it evaluates as
              `True`, a Notification for the Event is created and sent to the configured
              destination. Calculations can reference numeric, string, and boolean Event
              fields.

              See
              [Creating Calculations](https://www.m3ter.com/docs/guides/utilizing-events-and-notifications/key-concepts-and-relationships#creating-calculations)
              in the m3ter documentation for more information.

          version:
              The version number for the Notification:

              - **Create:** Not valid for initial insertion of new entity - _do not use for
                Create_. On initial Create, version is set at 1 and listed in the response.
              - **Update:** On Update, version is required and must match the existing version
                because a check is performed to ensure sequential versioning is preserved.
                Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/notifications/configurations/{id}",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "description": description,
                    "event_name": event_name,
                    "name": name,
                    "active": active,
                    "always_fire_event": always_fire_event,
                    "calculation": calculation,
                    "version": version,
                },
                notification_configuration_update_params.NotificationConfigurationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationConfigurationResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        active: bool | NotGiven = NOT_GIVEN,
        event_name: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[NotificationConfigurationResponse, AsyncCursor[NotificationConfigurationResponse]]:
        """
        Retrieve a list of Event Notifications for the specified Organization.

        This endpoint retrieves a list of all Event Notifications for the Organization
        identified by its UUID. The list can be paginated for easier management. The
        list also supports filtering by parameters such as Notification UUID.

        Args:
          active: A Boolean flag indicating whether to retrieve only active or only inactive
              Notifications.

              - **TRUE** - only active Notifications are returned.
              - **FALSE** - only inactive Notifications are returned.

          event_name: Use this to filter the Notifications returned - only those Notifications that
              are based on the _Event type_ specified by `eventName` are returned.

          ids: A list of specific Notification UUIDs to retrieve.

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              Notifications in a paginated list.

          page_size: Specifies the maximum number of Notifications to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/notifications/configurations",
            page=AsyncCursor[NotificationConfigurationResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "active": active,
                        "event_name": event_name,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    notification_configuration_list_params.NotificationConfigurationListParams,
                ),
            ),
            model=NotificationConfigurationResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> NotificationConfigurationResponse:
        """
        Delete the Notification with the given UUID.

        This endpoint permanently removes a specified Notification and its
        configuration. This action cannot be undone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/notifications/configurations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NotificationConfigurationResponse,
        )


class NotificationConfigurationsResourceWithRawResponse:
    def __init__(self, notification_configurations: NotificationConfigurationsResource) -> None:
        self._notification_configurations = notification_configurations

        self.create = to_raw_response_wrapper(
            notification_configurations.create,
        )
        self.retrieve = to_raw_response_wrapper(
            notification_configurations.retrieve,
        )
        self.update = to_raw_response_wrapper(
            notification_configurations.update,
        )
        self.list = to_raw_response_wrapper(
            notification_configurations.list,
        )
        self.delete = to_raw_response_wrapper(
            notification_configurations.delete,
        )


class AsyncNotificationConfigurationsResourceWithRawResponse:
    def __init__(self, notification_configurations: AsyncNotificationConfigurationsResource) -> None:
        self._notification_configurations = notification_configurations

        self.create = async_to_raw_response_wrapper(
            notification_configurations.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            notification_configurations.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            notification_configurations.update,
        )
        self.list = async_to_raw_response_wrapper(
            notification_configurations.list,
        )
        self.delete = async_to_raw_response_wrapper(
            notification_configurations.delete,
        )


class NotificationConfigurationsResourceWithStreamingResponse:
    def __init__(self, notification_configurations: NotificationConfigurationsResource) -> None:
        self._notification_configurations = notification_configurations

        self.create = to_streamed_response_wrapper(
            notification_configurations.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            notification_configurations.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            notification_configurations.update,
        )
        self.list = to_streamed_response_wrapper(
            notification_configurations.list,
        )
        self.delete = to_streamed_response_wrapper(
            notification_configurations.delete,
        )


class AsyncNotificationConfigurationsResourceWithStreamingResponse:
    def __init__(self, notification_configurations: AsyncNotificationConfigurationsResource) -> None:
        self._notification_configurations = notification_configurations

        self.create = async_to_streamed_response_wrapper(
            notification_configurations.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            notification_configurations.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            notification_configurations.update,
        )
        self.list = async_to_streamed_response_wrapper(
            notification_configurations.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            notification_configurations.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["TransactionCreateParams"]


class TransactionCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    amount: Required[float]
    """The financial value of the transaction."""

    applied_date: Annotated[Union[str, datetime], PropertyInfo(alias="appliedDate", format="iso8601")]
    """The date _(in ISO 8601 format)_ when the Balance transaction was applied."""

    currency_paid: Annotated[str, PropertyInfo(alias="currencyPaid")]
    """The currency code of the payment if it differs from the Balance currency.

    For example: USD, GBP or EUR.
    """

    description: str
    """A brief description explaining the purpose and context of the transaction."""

    paid: float
    """The payment amount if the payment currency differs from the Balance currency."""

    transaction_date: Annotated[Union[str, datetime], PropertyInfo(alias="transactionDate", format="iso8601")]
    """The date _(in ISO 8601 format)_ when the transaction occurred."""

    transaction_type_id: Annotated[str, PropertyInfo(alias="transactionTypeId")]
    """The unique identifier (UUID) of the transaction type.

    This is obtained from the list of created Transaction Types within the
    Organization Configuration.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import date, datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.currency_conversion import CurrencyConversion

__all__ = ["BillResponse", "LineItem", "LineItemUsagePerPricingBand"]


class LineItemUsagePerPricingBand(BaseModel):
    band_quantity: Optional[float] = FieldInfo(alias="bandQuantity", default=None)
    """Usage amount within the band."""

    band_subtotal: Optional[float] = FieldInfo(alias="bandSubtotal", default=None)
    """Subtotal amount for the band."""

    band_units: Optional[float] = FieldInfo(alias="bandUnits", default=None)
    """The number of units used within the band."""

    credit_type_id: Optional[str] = FieldInfo(alias="creditTypeId", default=None)
    """The UUID of the credit type."""

    fixed_price: Optional[float] = FieldInfo(alias="fixedPrice", default=None)
    """
    Fixed price is a charge entered for certain pricing types such as Stairstep,
    Custom Tiered, and Custom Volume. It is a set price and not dependent on usage.
    """

    lower_limit: Optional[float] = FieldInfo(alias="lowerLimit", default=None)
    """The lower limit _(start)_ of the pricing band."""

    pricing_band_id: Optional[str] = FieldInfo(alias="pricingBandId", default=None)
    """The UUID for the pricing band."""

    unit_price: Optional[float] = FieldInfo(alias="unitPrice", default=None)
    """The price per unit in the band."""

    unit_subtotal: Optional[float] = FieldInfo(alias="unitSubtotal", default=None)
    """The subtotal of the unit usage."""


class LineItem(BaseModel):
    average_unit_price: float = FieldInfo(alias="averageUnitPrice")
    """The average unit price across all tiers / pricing bands."""

    conversion_rate: float = FieldInfo(alias="conversionRate")
    """
    The currency conversion rate if currency conversion is required for the line
    item.
    """

    converted_subtotal: float = FieldInfo(alias="convertedSubtotal")
    """The converted subtotal amount if currency conversions have been used."""

    currency: str
    """The currency code for the currency used in the line item.

    For example: USD, GBP, or EUR.
    """

    description: str
    """Line item description."""

    line_item_type: Literal[
        "STANDING_CHARGE",
        "USAGE",
        "COUNTER_RUNNING_TOTAL_CHARGE",
        "COUNTER_ADJUSTMENT_DEBIT",
        "COUNTER_ADJUSTMENT_CREDIT",
        "USAGE_CREDIT",
        "MINIMUM_SPEND",
        "MINIMUM_SPEND_REFUND",
        "CREDIT_DEDUCTION",
        "MANUAL_ADJUSTMENT",
        "CREDIT_MEMO",
        "DEBIT_MEMO",
        "COMMITMENT_CONSUMED",
        "COMMITMENT_FEE",
        "OVERAGE_SURCHARGE",
        "OVERAGE_USAGE",
        "BALANCE_CONSUMED",
        "BALANCE_FEE",
    ] = FieldInfo(alias="lineItemType")

    quantity: float
    """The amount of usage for the line item."""

    subtotal: float
    """The subtotal amount for the line item, before any currency conversions."""

    unit: str
    """The unit for the usage data in thie line item.

    For example: **GB** of disk storage space.
    """

    units: float
    """The number of units used for the line item."""

    id: Optional[str] = None
    """The UUID for the line item."""

    aggregation_id: Optional[str] = FieldInfo(alias="aggregationId", default=None)
    """The Aggregation ID used for the line item."""

    balance_id: Optional[str] = FieldInfo(alias="balanceId", default=None)

    charge_id: Optional[str] = FieldInfo(alias="chargeId", default=None)

    child_account_code: Optional[str] = FieldInfo(alias="childAccountCode", default=None)
    """
    If part of a Parent/Child account billing hierarchy, this is the code for the
    child Account.
    """

    child_account_id: Optional[str] = FieldInfo(alias="childAccountId", default=None)
    """
    If part of a Parent/Child account billing hierarchy, this is the child Account
    UUID.
    """

    commitment_id: Optional[str] = FieldInfo(alias="commitmentId", default=None)
    """
    If Commitments _(prepayments)_ are used in the line item, this shows the
    Commitment UUID.
    """

    compound_aggregation_id: Optional[str] = FieldInfo(alias="compoundAggregationId", default=None)
    """
    The Compound Aggregation ID for the line item if a Compound Aggregation has been
    used.
    """

    contract_id: Optional[str] = FieldInfo(alias="contractId", default=None)
    """The UUID for the Contract used in the line item."""

    counter_id: Optional[str] = FieldInfo(alias="counterId", default=None)

    credit_type_id: Optional[str] = FieldInfo(alias="creditTypeId", default=None)

    group: Optional[Dict[str, str]] = None

    meter_id: Optional[str] = FieldInfo(alias="meterId", default=None)
    """The UUID of the Meter used in the line item."""

    plan_group_id: Optional[str] = FieldInfo(alias="planGroupId", default=None)
    """The UUID of the PlanGroup, provided the line item used a PlanGroup."""

    plan_id: Optional[str] = FieldInfo(alias="planId", default=None)
    """The ID of the Plan used for the line item."""

    pricing_id: Optional[str] = FieldInfo(alias="pricingId", default=None)
    """The UUID of the Pricing used on the line item."""

    product_code: Optional[str] = FieldInfo(alias="productCode", default=None)

    product_id: Optional[str] = FieldInfo(alias="productId", default=None)
    """The UUID of the Product for the line item."""

    product_name: Optional[str] = FieldInfo(alias="productName", default=None)
    """The name of the Product for the line item."""

    reason_id: Optional[str] = FieldInfo(alias="reasonId", default=None)

    referenced_bill_id: Optional[str] = FieldInfo(alias="referencedBillId", default=None)

    referenced_line_item_id: Optional[str] = FieldInfo(alias="referencedLineItemId", default=None)

    segment: Optional[Dict[str, str]] = None
    """Applies only when segmented Aggregations have been used.

    The Segment to which the usage data in this line item belongs.
    """

    sequence_number: Optional[int] = FieldInfo(alias="sequenceNumber", default=None)
    """The number used for sequential invoices."""

    service_period_end_date: Optional[datetime] = FieldInfo(alias="servicePeriodEndDate", default=None)
    """The ending date _(exclusive)_ for the service period _(in ISO 8601 format)_."""

    service_period_start_date: Optional[datetime] = FieldInfo(alias="servicePeriodStartDate", default=None)
    """The starting date _(inclusive)_ for the service period _(in ISO 8601 format)_."""

    usage_per_pricing_band: Optional[List[LineItemUsagePerPricingBand]] = FieldInfo(
        alias="usagePerPricingBand", default=None
    )
    """Shows the usage by pricing band for tiered pricing structures."""


class BillResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    account_code: Optional[str] = FieldInfo(alias="accountCode", default=None)

    account_id: Optional[str] = FieldInfo(alias="accountId", default=None)

    bill_date: Optional[date] = FieldInfo(alias="billDate", default=None)

    bill_frequency_interval: Optional[int] = FieldInfo(alias="billFrequencyInterval", default=None)

    billing_frequency: Optional[Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY", "AD_HOC", "MIXED"]] = FieldInfo(
        alias="billingFrequency", default=None
    )

    bill_job_id: Optional[str] = FieldInfo(alias="billJobId", default=None)

    bill_total: Optional[float] = FieldInfo(alias="billTotal", default=None)
    """The sum total for the Bill."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The unique identifier (UUID) for the user who created the Bill."""

    created_date: Optional[datetime] = FieldInfo(alias="createdDate", default=None)

    csv_statement_generated: Optional[bool] = FieldInfo(alias="csvStatementGenerated", default=None)
    """
    Flag to indicate that the statement in CSV format has been generated for the
    Bill.

    - **TRUE** - CSV statement has been generated.
    - **FALSE** - no CSV statement generated.
    """

    currency: Optional[str] = None

    currency_conversions: Optional[List[CurrencyConversion]] = FieldInfo(alias="currencyConversions", default=None)

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The date and time _(in ISO 8601 format)_ when the Bill was first created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The date and time _(in ISO 8601 format)_ when the Bill was last modified."""

    due_date: Optional[date] = FieldInfo(alias="dueDate", default=None)

    end_date: Optional[date] = FieldInfo(alias="endDate", default=None)

    end_date_time_utc: Optional[datetime] = FieldInfo(alias="endDateTimeUTC", default=None)

    external_invoice_date: Optional[date] = FieldInfo(alias="externalInvoiceDate", default=None)
    """
    For accounting purposes, the date set at Organization level to use for external
    invoicing with respect to billing periods - two options:

    - `FIRST_DAY_OF_NEXT_PERIOD` _(Default)_.
    - `LAST_DAY_OF_ARREARS`

    For example, if the retrieved Bill was on a monthly billing frequency and the
    billing period for the Bill is September 2023 and the _External invoice date_ is
    set at `FIRST_DAY_OF_NEXT_PERIOD`, then the `externalInvoiceDate` will be
    `"2023-10-01"`.

    **NOTE:** To change the `externalInvoiceDate` setting for your Organization, you
    can use the
    [Update OrganizationConfig](https://www.m3ter.com/docs/api#tag/OrganizationConfig/operation/GetOrganizationConfig)
    call.
    """

    external_invoice_reference: Optional[str] = FieldInfo(alias="externalInvoiceReference", default=None)
    """The reference ID to use for external invoice."""

    json_statement_generated: Optional[bool] = FieldInfo(alias="jsonStatementGenerated", default=None)
    """
    Flag to indicate that the statement in JSON format has been generated for the
    Bill.

    - **TRUE** - JSON statement has been generated.
    - **FALSE** - no JSON statement generated.
    """

    last_calculated_date: Optional[datetime] = FieldInfo(alias="lastCalculatedDate", default=None)

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The unique identifier (UUID) for the user who last modified this Bill."""

    line_items: Optional[List[LineItem]] = FieldInfo(alias="lineItems", default=None)
    """An array of the Bill line items."""

    locked: Optional[bool] = None

    purchase_order_number: Optional[str] = FieldInfo(alias="purchaseOrderNumber", default=None)
    """Purchase Order number linked to the Account the Bill is for."""

    sequential_invoice_number: Optional[str] = FieldInfo(alias="sequentialInvoiceNumber", default=None)
    """The sequential invoice number of the Bill.

    **NOTE:** If you have not defined a `billPrefix` for your Organization, a
    `sequentialInvoiceNumber` is not returned in the response. See
    [Update OrganizationConfig](https://www.m3ter.com/docs/api#tag/OrganizationConfig/operation/UpdateOrganizationConfig)
    """

    start_date: Optional[date] = FieldInfo(alias="startDate", default=None)

    start_date_time_utc: Optional[datetime] = FieldInfo(alias="startDateTimeUTC", default=None)

    status: Optional[Literal["PENDING", "APPROVED"]] = None

    timezone: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["IntegrationConfigurationDeleteResponse"]


class IntegrationConfigurationDeleteResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    destination: str
    """The type of destination _(e.g. Netsuite, webhooks)_."""

    entity_type: str = FieldInfo(alias="entityType")
    """The type of entity the integration is for _(e.g. Bill)_."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    authorized: Optional[bool] = None
    """A flag indicating whether the integration configuration is authorized.

    - TRUE - authorized.
    - FALSE - not authorized.
    """

    config_data: Optional[Dict[str, object]] = FieldInfo(alias="configData", default=None)
    """Configuration data for the integration"""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The ID of the user who created this item."""

    destination_id: Optional[str] = FieldInfo(alias="destinationId", default=None)
    """The unique identifier (UUID) of the entity the integration is for."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when this item was created _(in ISO-8601 format)_."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when this item was last modified _(in ISO-8601 format)_."""

    enabled: Optional[bool] = None
    """
    A flag indicating whether the integration configuration is currently enabled or
    disabled.

    - TRUE - enabled.
    - FALSE - disabled.
    """

    entity_id: Optional[str] = FieldInfo(alias="entityId", default=None)
    """The unique identifier (UUID) of the entity this integration is for \\**(e.g.

    the ID of a notification configuration. Optional.)\\**
    """

    integration_credentials_id: Optional[str] = FieldInfo(alias="integrationCredentialsId", default=None)
    """UUID of the credentials to use for this integration"""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The ID of the user who last modified this item."""

    name: Optional[str] = None
    """The name of the configuration"""

    trigger_type: Optional[Literal["EVENT", "SCHEDULE"]] = FieldInfo(alias="triggerType", default=None)
    """Specifies the type of trigger for the integration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["JobGetDownloadURLResponse"]


class JobGetDownloadURLResponse(BaseModel):
    expiration_time: Optional[datetime] = FieldInfo(alias="expirationTime", default=None)
    """The expiration time of the URL"""

    url: Optional[str] = None
    """The presigned download URL"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["JobListParams"]


class JobListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    date_created_end: Annotated[str, PropertyInfo(alias="dateCreatedEnd")]
    """Include only File Upload jobs created before this date.

    Required format is ISO-8601: yyyy-MM-dd'T'HH:mm:ss'Z'
    """

    date_created_start: Annotated[str, PropertyInfo(alias="dateCreatedStart")]
    """Include only File Upload jobs created on or after this date.

    Required format is ISO-8601: yyyy-MM-dd'T'HH:mm:ss'Z'
    """

    file_key: Annotated[Optional[str], PropertyInfo(alias="fileKey")]
    """<<deprecated>>"""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """`nextToken` for multi page retrievals."""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Number of File Upload jobs to retrieve per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ExternalMappingListParams"]


class ExternalMappingListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    external_system_id: Annotated[str, PropertyInfo(alias="externalSystemId")]
    """The name of the external system to use as a filter.

    For example, if you want to list only those external mappings created for your
    Organization for the Salesforce external system, use:

    `?externalSystemId=Salesforce`
    """

    integration_config_id: Annotated[str, PropertyInfo(alias="integrationConfigId")]
    """ID of the integration config"""

    m3ter_ids: Annotated[List[str], PropertyInfo(alias="m3terIds")]
    """IDs for m3ter entities"""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """The `nextToken` for multi-page retrievals.

    It is used to fetch the next page of External Mappings in a paginated list.
    """

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Specifies the maximum number of External Mappings to retrieve per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["IntegrationConfigurationUpdateParams", "Credentials"]


class IntegrationConfigurationUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    config_data: Required[Annotated[Dict[str, object], PropertyInfo(alias="configData")]]
    """
    A flexible object to include any additional configuration data specific to the
    integration.
    """

    credentials: Required[Credentials]
    """
    Base model for defining integration credentials across different types of
    integrations.
    """

    destination: Required[str]
    """Denotes the integration destination.

    This field identifies the target platform or service for the integration.
    """

    destination_id: Required[Annotated[str, PropertyInfo(alias="destinationId")]]
    """The unique identifier (UUID) for the integration destination."""

    entity_id: Required[Annotated[str, PropertyInfo(alias="entityId")]]
    """The unique identifier (UUID) of the entity.

    This field is used to specify which entity's integration configuration you're
    updating.
    """

    entity_type: Required[Annotated[str, PropertyInfo(alias="entityType")]]
    """
    Specifies the type of entity for which the integration configuration is being
    updated. Must be a valid alphanumeric string.
    """

    integration_credentials_id: Required[Annotated[str, PropertyInfo(alias="integrationCredentialsId")]]

    name: Required[str]

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """


class Credentials(TypedDict, total=False):
    type: Required[
        Literal[
            "HTTP_BASIC",
            "OAUTH_CLIENT_CREDENTIALS",
            "M3TER_SIGNED_REQUEST",
            "AWS_INTEGRATION",
            "PADDLE_AUTH",
            "NETSUITE_AUTH",
            "CHARGEBEE_AUTH",
            "M3TER_SERVICE_USER",
            "STRIPE_SIGNED_REQUEST",
        ]
    ]
    """Specifies the type of authorization required for the integration."""

    destination: Literal[
        "WEBHOOK",
        "NETSUITE",
        "STRIPE",
        "STRIPE_TEST",
        "AWS",
        "PADDLE",
        "PADDLE_SANDBOX",
        "SALESFORCE",
        "XERO",
        "CHARGEBEE",
        "QUICKBOOKS",
        "QUICKBOOKS_SANDBOX",
        "M3TER",
    ]

    empty: bool
    """A flag to indicate whether the credentials are empty.

    - TRUE - empty credentials.
    - FALSE - credential details required.
    """

    name: str
    """The name of the credentials"""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo
from .permission_statement_response_param import PermissionStatementResponseParam

__all__ = ["UserUpdateParams"]


class UserUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    dt_end_access: Annotated[Union[str, datetime], PropertyInfo(alias="dtEndAccess", format="iso8601")]
    """The date and time _(in ISO 8601 format)_ when the user's access will end.

    Use this to set or update the expiration of the user's access.
    """

    permission_policy: Annotated[Iterable[PermissionStatementResponseParam], PropertyInfo(alias="permissionPolicy")]
    """An array of permission statements for the user.

    Each permission statement defines a specific permission for the user.

    See
    [Understanding, Creating, and Managing Permission Policies](https://www.m3ter.com/docs/guides/organization-and-access-management/creating-and-managing-permissions)
    for more information.
    """

    version: int
    """The version number of the entity:

    - **Newly created entity:** On initial Create, version is set at 1 and listed in
      the response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["PermissionStatementResponse"]


class PermissionStatementResponse(BaseModel):
    action: List[
        Literal[
            "ALL",
            "CONFIG_CREATE",
            "CONFIG_RETRIEVE",
            "CONFIG_UPDATE",
            "CONFIG_DELETE",
            "CONFIG_EXPORT",
            "ANALYTICS_QUERY",
            "MEASUREMENTS_UPLOAD",
            "MEASUREMENTS_FILEUPLOAD",
            "MEASUREMENTS_RETRIEVE",
            "MEASUREMENTS_EXPORT",
            "FORECAST_RETRIEVE",
            "HEALTHSCORES_RETRIEVE",
            "ANOMALIES_RETRIEVE",
            "EXPORTS_DOWNLOAD",
        ]
    ]
    """
    The actions available to users who are assigned the Permission Policy - what
    they can do or cannot do with respect to the specified resource.

    **NOTE:** Use lower case and a colon-separated format, for example, if you want
    to confer full CRUD, use:

    ```
    "config:create",
    "config:delete",
    "config:retrieve",
    "config:update"
    ```
    """

    effect: Literal["ALLOW", "DENY"]
    """
    Specifies whether or not the user is allowed to perform the action on the
    resource.

    **NOTE:** Use lower case, for example: `"allow"`. If you use upper case, you'll
    receive an error.
    """

    resource: List[str]
    """
    See
    [Statements - Available Resources](https://www.m3ter.com/docs/guides/managing-organization-and-users/creating-and-managing-permissions#statements---available-resources)
    for a listing of available resources for Permission Policy statements.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["UserMeResponse", "Organization", "ServiceUser", "User"]


class Organization(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    address_line1: Optional[str] = FieldInfo(alias="addressLine1", default=None)

    address_line2: Optional[str] = FieldInfo(alias="addressLine2", default=None)

    address_line3: Optional[str] = FieldInfo(alias="addressLine3", default=None)

    address_line4: Optional[str] = FieldInfo(alias="addressLine4", default=None)

    billing_contact_user_id1: Optional[str] = FieldInfo(alias="billingContactUserId1", default=None)

    billing_contact_user_id2: Optional[str] = FieldInfo(alias="billingContactUserId2", default=None)

    billing_contact_user_id3: Optional[str] = FieldInfo(alias="billingContactUserId3", default=None)

    country: Optional[str] = None

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The id of the user who created this organization."""

    customer_id: Optional[str] = FieldInfo(alias="customerId", default=None)

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when the organization was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when the organization was last modified."""

    invoice_general_reference: Optional[str] = FieldInfo(alias="invoiceGeneralReference", default=None)

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The id of the user who last modified this organization."""

    locality: Optional[str] = None

    organization_name: Optional[str] = FieldInfo(alias="organizationName", default=None)

    org_id: Optional[str] = FieldInfo(alias="orgId", default=None)

    post_code: Optional[str] = FieldInfo(alias="postCode", default=None)

    purchase_order_number: Optional[str] = FieldInfo(alias="purchaseOrderNumber", default=None)

    region: Optional[str] = None

    short_name: Optional[str] = FieldInfo(alias="shortName", default=None)

    status: Optional[Literal["ACTIVE", "ARCHIVED"]] = None

    tax_id: Optional[str] = FieldInfo(alias="taxId", default=None)

    type: Optional[Literal["PRODUCTION", "SANDBOX"]] = None


class ServiceUser(BaseModel):
    id: Optional[str] = None

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The id of the user who created this service user."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when the service user was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when the service user was last modified."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The id of the user who last modified this service user."""

    name: Optional[str] = None

    version: Optional[int] = None


class User(BaseModel):
    id: Optional[str] = None
    """The unique identifier (UUID) of this user."""

    contact_number: Optional[str] = FieldInfo(alias="contactNumber", default=None)
    """The user's contact telephone number."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The user who created this user."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The date and time _(in ISO-8601 format)_ when the user was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The date and time _(in ISO-8601 format)_ when the user was last modified."""

    email: Optional[str] = None
    """The email address for this user."""

    first_accepted_terms_and_conditions: Optional[datetime] = FieldInfo(
        alias="firstAcceptedTermsAndConditions", default=None
    )
    """
    The date and time _(in ISO 8601 format)_ when this user first accepted the the
    m3ter terms and conditions.
    """

    first_name: Optional[str] = FieldInfo(alias="firstName", default=None)
    """The first name of the user."""

    last_accepted_terms_and_conditions: Optional[datetime] = FieldInfo(
        alias="lastAcceptedTermsAndConditions", default=None
    )
    """
    The date and time _(in ISO 8601 format)_ when this user last accepted the the
    m3ter terms and conditions.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The unique identifier (UUID) of the user who last modified this user record."""

    last_name: Optional[str] = FieldInfo(alias="lastName", default=None)
    """The surname of the user."""

    organizations: Optional[List[str]] = None
    """An array listing the Organizations where this user has access."""

    support_user: Optional[bool] = FieldInfo(alias="supportUser", default=None)
    """Indicates whether this is a m3ter Support user."""

    version: Optional[int] = None
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """


class UserMeResponse(BaseModel):
    organization: Optional[Organization] = None

    service_user: Optional[ServiceUser] = FieldInfo(alias="serviceUser", default=None)

    user: Optional[User] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PlanTemplateListParams"]


class PlanTemplateListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    ids: List[str]
    """List of specific PlanTemplate UUIDs to retrieve."""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """The `nextToken` for multi-page retrievals.

    It is used to fetch the next page of PlanTemplates in a paginated list.
    """

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Specifies the maximum number of PlanTemplates to retrieve per page."""

    product_id: Annotated[str, PropertyInfo(alias="productId")]
    """
    The unique identifiers (UUIDs) of the Products to retrieve associated
    PlanTemplates.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.set_string import SetString

__all__ = ["ContractEndDateBillingEntitiesResponse", "FailedEntities", "UpdatedEntities"]


class FailedEntities(BaseModel):
    accountplan: Optional[SetString] = FieldInfo(alias="ACCOUNTPLAN", default=None)

    contract: Optional[SetString] = FieldInfo(alias="CONTRACT", default=None)

    counter_pricings: Optional[SetString] = FieldInfo(alias="COUNTER_PRICINGS", default=None)

    prepayment: Optional[SetString] = FieldInfo(alias="PREPAYMENT", default=None)

    pricings: Optional[SetString] = FieldInfo(alias="PRICINGS", default=None)


class UpdatedEntities(BaseModel):
    accountplan: Optional[SetString] = FieldInfo(alias="ACCOUNTPLAN", default=None)

    contract: Optional[SetString] = FieldInfo(alias="CONTRACT", default=None)

    counter_pricings: Optional[SetString] = FieldInfo(alias="COUNTER_PRICINGS", default=None)

    prepayment: Optional[SetString] = FieldInfo(alias="PREPAYMENT", default=None)

    pricings: Optional[SetString] = FieldInfo(alias="PRICINGS", default=None)


class ContractEndDateBillingEntitiesResponse(BaseModel):
    failed_entities: Optional[FailedEntities] = FieldInfo(alias="failedEntities", default=None)
    """
    A dictionary with keys as identifiers of billing entities and values as lists
    containing details of the entities for which the update failed.
    """

    status_message: Optional[str] = FieldInfo(alias="statusMessage", default=None)
    """A message indicating the status of the operation."""

    updated_entities: Optional[UpdatedEntities] = FieldInfo(alias="updatedEntities", default=None)
    """
    A dictionary with keys as identifiers of billing entities and values as lists
    containing details of the updated entities.
    """
